from ak_file.sanitize import *

def test_blacklist():
    assert sanitize("a\\b/c:d*e?f\"g<h>i|j\0k") == "abcdefghijk"

def test_reserved():
    for each in reserved:
        assert sanitize(each) != each

def test_length():
    assert len(sanitize("gradsf"*250)) == 255
    assert len(sanitize("gra:d\\sf"*250)) == 255

def test_special():
    assert sanitize("") == '__'


#!/usr/bin/env python
# coding: utf-8

from time import ctime
from pathlib import Path

class File:
    def __init__(self, filepath: str)-> None:
        self.filepath=Path(str(filepath))

    def __str__(self) -> str:
        file_prop = self.get_file_properties()
        if self.exists():
            return f"""
Name        : {self.name}
Directory   : {self.parent}
Exists          : True
Access Time     : {file_prop["Access Time"]}
Modified Time   : {file_prop["Modified Time"]}
Change Time     : {file_prop["Change Time"]}
Size            : {file_prop["Size_B"]/1024:.2f} KB
            """
        else:
            return f"""
Name        : {self.name}
Directory   : {self.parent}
Exists          : False
            """

    def __repr__(self) -> str:
        return f'File(filepath="{self.filepath}")'

    @property
    def name(self) -> str:
        "Returns the filename from the current self.filepath"
        return self.filepath.name

    @property
    def parent(self) -> Path:
        "Returns the file directory from the current self.filepath"
        return self.filepath.absolute().parent
    
    @property
    def abspath(self) -> Path:
        "Returns absolute path of the current file"
        return self.filepath.absolute()

    def update_filename(self, new_filename:str, rename_file: bool = False) -> str:
        "Updates the filepath of the stored file"
        new_filepath = self.filepath.parent / new_filename
        if rename_file and self.exists():
            self.filepath.rename(new_filepath)
        self.filepath = new_filepath
        return self.name
    
    def is_file(self) -> bool:
        return self.filepath.is_file()

    def is_dir(self) -> bool:
        return self.filepath.is_dir()

    def exists(self) -> bool:
        return self.filepath.is_file() or self.filepath.is_dir()

    def properties(self) -> dict:
        "Returns a dict of properties of the file"
        stat = self.filepath.stat() if self.exists() else None
        return {
            "name": self.name,
            "dir": self.parent,
            "exists": self.exists(),
            "Access Time": ctime(stat.st_atime) if stat else None,
            "Modified Time": ctime(stat.st_mtime) if stat else None,
            "Change Time" : ctime(stat.st_ctime) if stat else None,
            "Size_B": stat.st_size if stat else None
        }










from ak_file.main import *
from pathlib import Path

root = Path(__file__).parent.parent

class Test_File:
    file1 = File(root / "README.md")
    folder1 = File(root / "tests")
    file2 = File(root / "ak_file" / "__init__.py")
    folder2 = File(root / "ak_file")

    def test_name(self):
        assert self.file1.name == "README.md"
        assert self.folder1.name == "tests"
        assert self.file2.name == "__init__.py"
        assert self.folder2.name == "ak_file"

    def test_parent(self):
        assert self.file2.parent.name == "ak_file"
        assert self.file1.parent == self.folder1.parent
        assert self.file1.parent == self.folder2.parent
    
    def test_is_file(self):
        assert self.file1.is_file() == True
        assert self.folder1.is_file() == False
        assert self.file2.is_file() == True
        assert self.folder1.is_file() == False

    def test_is_dir(self):
        assert self.file1.is_dir() == False
        assert self.folder1.is_dir() == True
        assert self.file2.is_dir() == False
        assert self.folder1.is_dir() == True

    def test_exists(self):
        assert self.file1.exists() == True
        assert self.folder1.exists() == True
        assert self.file2.exists() == True
        assert self.folder1.exists() == True


from pathlib import Path
from typing import Generator
import re

def by_extension(folder_path:str, extension:str, search_subdir:bool=False) -> Generator[Path, None, None]:
    "Returns a list of files in the directory with matching extension"
    return Path(str(folder_path)).glob(f"{'**/' if search_subdir else ''}*.{extension}")

def by_regex(
    folder_path: str, 
    regex_pattern: str, 
    search_subdir:bool=False, 
    case_sensitive:bool=False) -> list[Path]:

    folder_path = Path(str(folder_path))

    if (not case_sensitive) and (not regex_pattern.startswith("(?i)")):
        regex_pattern = "(?i)" + regex_pattern
    
    regex = re.compile(regex_pattern)

    files = folder_path.glob(f"{'**/' if search_subdir else ''}*")
    return [ file for file in files if regex.search(file.name)]



import unicodedata, re

blacklist = ["\\", "/", ":", "*", "?", "\"", "<", ">", "|", "\0"]

# Reserved words on Windows
reserved = [
    "CON", "PRN", "AUX", "NUL", "COM1", "COM2", "COM3", "COM4", "COM5",
    "COM6", "COM7", "COM8", "COM9", "LPT1", "LPT2", "LPT3", "LPT4", "LPT5",
    "LPT6", "LPT7", "LPT8", "LPT9",
]  

def sanitize(filename:str) -> str:
    """Return a fairly safe version of the filename.

    We don't limit ourselves to ascii, because we want to keep municipality
    names, etc, but we do want to get rid of anything potentially harmful,
    and make sure we do not exceed Windows filename length limits.
    Hence a less safe blacklist, rather than a whitelist.
    """
    
    filename = "".join(c for c in filename if c not in blacklist)
    # Remove all charcters below code point 32
    filename = "".join(c for c in filename if 31 < ord(c))
    filename = unicodedata.normalize("NFKD", filename)
    filename = filename.rstrip(". ")  # Windows does not allow these at end
    filename = filename.strip()
    if all([x == "." for x in filename]):
        filename = "__" + filename
    if filename in reserved:
        filename = "__" + filename
    if len(filename) == 0:
        filename = "__"
    if len(filename) > 255:
        parts = re.split(r"/|\\", filename)[-1].split(".")
        if len(parts) > 1:
            ext = "." + parts.pop()
            filename = filename[:-len(ext)]
        else:
            ext = ""
        if filename == "":
            filename = "__"
        if len(ext) > 254:
            ext = ext[254:]
        maxl = 255 - len(ext)
        filename = filename[:maxl]
        filename = filename + ext
        # Re-check last character (if there was no extension)
        filename = filename.rstrip(". ")
        if len(filename) == 0:
            filename = "__"
    return filename


from ak_file.search import *
from pathlib import Path

parent_dir = Path(__file__).parent.parent

def test_by_extension():
    assert len(list(by_extension(parent_dir, 'py', False))) == 0
    assert len(list(by_extension(parent_dir, 'toml', False))) == 1

def test_by_regex():
    assert len(by_regex(parent_dir, r'.*.PY$',True, True)) == 0
    assert len(by_regex(parent_dir, r'.*.PY$',True, False)) > 0